<?php
/**
 * Project: ServerSync
 * User: Smile
 * Date: 10/15/2019
 * Time: 11:55 PM
 */


abstract class PullPush
{
    /** @var MysqliDb */
    protected $server;
    /** @var MysqliDb */
    protected $local;

    protected $lock_local = [];
    protected $lock_server = [];

    protected $db_type_local = self::DB_TYPE_DEFAULT;
    protected $db_type_server = self::DB_TYPE_DEFAULT;

    protected $sync_type = self::SYNC_TYPE_PULL;

    protected $last_sync_time = 1;

    const SYNC_TYPE_PUSH = 'push';
    const SYNC_TYPE_PULL = 'pull';

    const DB_TYPE_MASTER = 'master';
    const DB_TYPE_DEFAULT = 'default';

    /** @var App */
    public static $app = null;

    public function __construct()
    {
        $this->server = self::$app->db_server;
        $this->local  = self::$app->db_local;
    }

    /**
     * @throws Exception
     */
    public function sync()
    {
        $db_name_server = self::$app->db_conf_server['Database'];
        $db_name_local  = self::$app->db_conf_local['Database'];

        if ($this->db_type_server === self::DB_TYPE_MASTER)
            $db_name_server = self::$app->db_conf_server_master['Database'];

        if ($this->db_type_local === self::DB_TYPE_MASTER)
            $db_name_local = self::$app->db_conf_local_master['Database'];

        $this->server->use($db_name_server);
        $this->local->use($db_name_local);

        $this->log(static::class . ' :: STARTED');

        if ($this->sync_type === self::SYNC_TYPE_PULL) {
            $this->syncPull();
        }
        elseif ($this->sync_type === self::SYNC_TYPE_PUSH) {
            $this->syncPush();
        }
    }

    /**
     * @throws Exception
     */
    private function syncPull()
    {
        $this->last_sync_time = date("Y-m-d H:i:s", $this->getLastSyncTime());

        $this->lockServer('read');
        $sync_time = time();
        $data = $this->loadItems();
        $this->unlockServer();


        $this->lockLocal('write');
        $this->local->startTransaction();
        $this->saveItems($data);
        $this->local->commit();

        $this->unlockLocal();

        $this->setSyncTime($sync_time);
    }

    /**
     * @throws Exception
     */
    private function syncPush()
    {
        $this->last_sync_time = date("Y-m-d H:i:s", $this->getLastSyncTime());

        $this->lockLocal('read');

        $sync_time = time();

        $data = $this->loadItems();

        $this->unlockLocal();

        $this->lockServer('write');

        $this->local->startTransaction();
        $this->saveItems($data);
        $this->local->commit();

        $this->unlockServer();

        $this->setSyncTime($sync_time);
    }


    /**
     * @param $method
     * @throws Exception
     */
    private function lockServer($method)
    {
        $this->server
            ->setLockMethod($method)
            ->lock($this->lock_server);
    }

    /**
     * @throws Exception
     */
    private function unlockServer()
    {
        $this->server->unlock();
    }

    /**
     * @param $method
     * @throws Exception
     */
    private function lockLocal($method)
    {
        $this->local
            ->setLockMethod($method)
            ->lock($this->lock_local);
    }

    /**
     * @throws Exception
     */
    private function unlockLocal()
    {
        $this->local->unlock();
    }

    /**
     * @throws Exception
     */
    protected abstract function loadItems();

    /**
     * @param $data
     * @throws Exception
     */
    protected abstract function saveItems($data);

    private function getLastSyncTime()
    {
        return self::$app->GetStoredValue(static::class, 1111);
    }

    private function setSyncTime($time)
    {
        self::$app->SetStoredValue(static::class, $time);
    }

    /**
     * @throws Exception
     */
    protected function throwExceptionOnLocalError()
    {
        if ($this->local->getLastErrno() != 0) {
            throw new Exception('Local ' . $this->local->getLastError(), 500);
        }
    }

    /**
     * @throws Exception
     */
    protected function throwExceptionOnServerError()
    {
        if ($this->server->getLastErrno() != 0) {
            throw new Exception('Server: ' . $this->server->getLastError(), 500);
        }
    }

    /**
     * @throws Exception
     */
    protected function throwNoDataException()
    {
        throw new Exception(static::class . ' :: NO DATA', 404);
    }

    protected function log($message)
    {
        self::$app->log($message);
    }

    protected function logError($message)
    {
        self::$app->logError($message);
    }
}
